/*
 * TextProvider.java # 2003-9-28
 * mailto: shenchilang@163.com
 */
package cn.edu.nudt.mice.italk.content;


import java.awt.Toolkit;
import java.io.IOException;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.Separator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IActionBars;

import cn.edu.nudt.mice.italk.Resources;
import cn.edu.nudt.mice.italk.content.actions.BeepAction;
import cn.edu.nudt.mice.italk.content.actions.ClearAction;
import cn.edu.nudt.mice.italk.content.actions.DeleteAction;
import cn.edu.nudt.mice.italk.content.actions.InsertAction;
import cn.edu.nudt.mice.italk.views.TalkView;
import cn.nudt.edu.mice.italk.peer.TalkPeer;

/**
 * @author Shen Rui
 * <code>TextProvider</code>
 */
public class TextProvider implements 
		MouseListener, KeyListener, ModifyListener, SelectionListener{
	
	public static final String PROVIDER_KEY = "provider.text";		
	
	public TextProvider(TalkView view, Composite parent, int style){
		_view = view;
		_text = new Text(parent, style);
		_text.setEditable(false);
		_text.addMouseListener(this);
		_text.addKeyListener(this);
		_text.addModifyListener(this);
		_text.addSelectionListener(this);
		
		makeActions();
	}
	
	private void makeActions(){
		{
			_close = new Action(){
				public void run(){
					if (_peer != null)
						_peer.close();
				}
			};
			_close.setText("Close");
			_close.setToolTipText("close current session");
			_close.setImageDescriptor(Resources.IMAGE_STOP);
			_close.setDisabledImageDescriptor(Resources.IMAGE_STOP_D);
			_close.setEnabled(false);
			_close.setAccelerator(SWT.ALT | 'X');
		}
		{
			_beep = new Action(){
				public void run(){
					send(new BeepAction());
				}
			};
			_beep.setText("Beep");
			_beep.setToolTipText("( ( ( Beep ) ) )");
			_beep.setImageDescriptor(Resources.IMAGE_BEEP);
			_beep.setDisabledImageDescriptor(Resources.IMAGE_BEEP_D);
			_beep.setEnabled(false);
			_beep.setAccelerator(SWT.CTRL | 'B');
		}
		{
			_clear = new Action(){
				public void run(){
					send(new ClearAction());
					_text.setText("");
				}
			};
			_clear.setText("Clear");
			_clear.setToolTipText("clear text");
			_clear.setImageDescriptor(Resources.IMAGE_CLEAR);
			_clear.setDisabledImageDescriptor(Resources.IMAGE_CLEAR_D);
			_clear.setEnabled(false);
			_clear.setAccelerator(SWT.CTRL | 'F');
		}
		IActionBars bars = _view.getViewSite().getActionBars();
		bars.getMenuManager().add(new Separator());
		bars.getMenuManager().add(_close);

		bars.getToolBarManager().add(new Separator());		
		bars.getToolBarManager().add(_beep);
		bars.getToolBarManager().add(_clear);
		bars.getToolBarManager().add(_close);
	}

	/**
	 * bind new peer to this provider
	 * MUST be execute under Display.syncExec()
	 * @safe_thread 
	 */
	public void _bind(TalkPeer peer){
		// save current data
		if (_peer != null){
			_peer.setData(PROVIDER_KEY, _text.getText());
			_text.setEditable(false);
			_close.setEnabled(false);
			_beep.setEnabled(false);
			_clear.setEnabled(false);
		}
		
		_peer = null;	// be sure _text.setText("") won't send ClearAction
		_text.setText("");
		
		// load new data
		if (peer != null){
			String s = (String) peer.getData(PROVIDER_KEY);
			if (s == null)
				s = "";
			_text.setText(s);
			_text.setEditable(true);
			_close.setEnabled(true);
			_beep.setEnabled(true);
			_clear.setEnabled(true);
			_text.setFocus();
		}		
		_peer = peer;
		
	}
	
	
	public TalkPeer getPeer(){
		return _peer;
	}
	
	private void updateCaretPosition(){
		_last_caret_pos = _text.getCaretPosition();
	}
	
	private void send(TextAction action){
		try {
			if (_peer != null)
				_peer.send(action);
		} catch (IOException e) {
		}
	}
	
	private void fireUpdated(){
		if (_peer != null){
			String s = _text.getText();
			int pos = _text.getCaretPosition();
			if (pos > _last_caret_pos){
				send(new InsertAction(_last_caret_pos, s.substring(_last_caret_pos, pos)));
			}else if (pos < _last_caret_pos){
				send(new DeleteAction(pos, _last_caret_pos));					
			}else{
				send(new ClearAction());
				send(new InsertAction(0, s));
			}
		}
		// don't forget update caret !!! EVEN when _peer is null		
		updateCaretPosition();
	}
	
	public Text getControl(){
		return _text;
	}

	/**
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		fireUpdated();
	}

	/**
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent e) {
		
	}

	/**
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetDefaultSelected(SelectionEvent e) {
		
	}

	/**
	 * @see org.eclipse.swt.events.MouseListener#mouseDoubleClick(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseDoubleClick(MouseEvent e) {
		
	}

	/**
	 * @see org.eclipse.swt.events.MouseListener#mouseDown(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseDown(MouseEvent e) {
		
	}

	/**
	 * @see org.eclipse.swt.events.MouseListener#mouseUp(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseUp(MouseEvent e) {
		updateCaretPosition();
	}

	/**
	 * @see org.eclipse.swt.events.KeyListener#keyPressed(org.eclipse.swt.events.KeyEvent)
	 */
	public void keyPressed(KeyEvent e) {
		updateCaretPosition();
	}

	/**
	 * @see org.eclipse.swt.events.KeyListener#keyReleased(org.eclipse.swt.events.KeyEvent)
	 */
	public void keyReleased(KeyEvent e) {
		
	}
	
	private TalkView _view;
	private Text _text;
	private TalkPeer _peer;
	private int _last_caret_pos;
	
	private Action _close;
	private Action _beep;
	private Action _clear;
}
