/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package cn.edu.nudt.mice.italk.preferences;

import org.eclipse.jface.preference.*;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.IWorkbench;
import cn.edu.nudt.mice.italk.TalkPlugin;
import org.eclipse.jface.preference.IPreferenceStore;

/**
 * This class represents a preference page that
 * is contributed to the Preferences dialog. By 
 * subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows
 * us to create a page that is small and knows how to 
 * save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They
 * are stored in the preference store that belongs to
 * the main plug-in class. That way, preferences can
 * be accessed directly via the preference store.
 */

public class TalkPreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {
		
	public static final String P_SERVER_PORT = "server.port";
	public static final String P_SERVER_AUTO = "server.auto";

	public TalkPreferencePage() {
		super(GRID);
		setPreferenceStore(TalkPlugin.getDefault().getPreferenceStore());
		setDescription("iTalk Preference Page (You can send bug to shenchilang@163.com)");
		initializeDefaults();
		// TODO read error port when not visit the preference
	}
/**
 * Sets the default values of the preferences.
 */
	private void initializeDefaults() {
		IPreferenceStore store = getPreferenceStore();
		store.setDefault(P_SERVER_PORT, "2003");
		store.setDefault(P_SERVER_PORT, "false");
	}
	
/**
 * Creates the field editors. Field editors are abstractions of
 * the common GUI blocks needed to manipulate various types
 * of preferences. Each field editor knows how to save and
 * restore itself.
 */

	public void createFieldEditors() {
		addField(new BooleanFieldEditor(P_SERVER_AUTO, "Server &Auto Start", getFieldEditorParent()));
		addField(new IntegerFieldEditor(P_SERVER_PORT, "Server &Port:", getFieldEditorParent()));
	}
	
	public void init(IWorkbench workbench) {
		initializeDefaults();		
	}
}