/*
 * PeerList.java @ 2003-9-27
 * mailto: shenchilang@163.com
 */
package cn.edu.nudt.mice.italk.views;

import java.awt.Toolkit;
import java.io.IOException;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.ui.IActionBars;

import cn.edu.nudt.mice.italk.Resources;
import cn.nudt.edu.mice.italk.peer.TalkPeer;
import cn.nudt.edu.mice.italk.peer.TalkServerCallback;

/**
 * @author Shen Rui
 * <code>PeerList</code>
 */
public class PeerList extends Composite implements TalkServerCallback{

	public PeerList(TalkPane pane) {
		super(pane, SWT.BORDER);
		_pane = pane;
		
		setLayout(new FillLayout());
		_list = new List(this, SWT.V_SCROLL | SWT.H_SCROLL);
		_list.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent evt){
				int i = _list.getSelectionIndex();
				if (i>=0){
					String id = _list.getItem(i);					
					_pane._bind(_pane.getServer().get(id));
				}
			}
		});
		
		createActions();
	}
	
	private void createActions(){
		{		
			_connect = new Action(){
				public void run(){
					ConnectDialog cd = new ConnectDialog(getShell());
					cd.setBlockOnOpen(true);
					cd.open();
					if (cd.getReturnCode() == ConnectDialog.OK){
						try {
							TalkPeer peer = _pane.getServer().connect(cd.getHost());
							accepted(peer);
						} catch (IOException e) {
							MessageDialog
								.openError(getShell(), 
										"connect failure", 
										"due to " 
											+ e.getClass() 
											+ ": " + e.getLocalizedMessage());
							e.printStackTrace();
						}
					}
				}
			};
			_connect.setText("Connect ...");
			_connect.setToolTipText("connect to a host");
			_connect.setImageDescriptor(Resources.IMAGE_CONNECT);
			_connect.setDisabledImageDescriptor(Resources.IMAGE_CONNECT_D);
			_connect.setEnabled(true);
			_connect.setAccelerator(SWT.ALT | 'C');
		}

		IActionBars bars = _pane.getView().getViewSite().getActionBars();
		bars.getMenuManager().add(_connect);
		
		bars.getToolBarManager().add(_connect);
		
	}
	
	
	private void add(String s){
		_list.add(s);
		_list.setSelection(_list.getItemCount() - 1);
	}

	private void remove(String s){
		int i = 0;
		for(; i< _list.getItemCount(); i++){
			if (_list.getItem(i).equals(s)){
				_list.remove(i);
				break;
			}
		}
		if (i > 0) i --;
		if (_list.getItemCount() > i){
			String id = _list.getItem(i);
			_pane._bind(_pane.getServer().get(id));
			_list.setSelection(i);
		}else{
			_pane._bind(null);			
		}
	}

	/**
	 * @see cn.nudt.edu.mice.italk.peer.TalkServerCallback#accepted(cn.nudt.edu.mice.italk.peer.TalkPeer)
	 */
	public boolean accepted(final TalkPeer peer) {
		getDisplay().syncExec(new Runnable(){
			public void run(){
				synchronized(_pane){
					add(peer.getId());
					_pane._bind(peer);
				}
			}
		});
		Toolkit.getDefaultToolkit().beep();
		return true;
	}
	
	/**
	 * @see cn.nudt.edu.mice.italk.peer.TalkServerCallback#disconnected(cn.nudt.edu.mice.italk.peer.TalkPeer)
	 */
	public void disconnected(final TalkPeer peer) {
		getDisplay().syncExec(new Runnable(){
			public void run(){
				synchronized(_pane){
					int i = 0;
					for(; i< _list.getItemCount(); i++){
						if (_list.getItem(i).equals(peer.getId())){
							if (i == _list.getSelectionIndex()){
								remove(peer.getId());
							}else{
								_list.remove(i);
							}
							break;
						}
					}
				}
			}
		});
		Toolkit.getDefaultToolkit().beep();
	}

	private TalkPane _pane;
	private List _list;

	private Action _connect;	
}
