/*
 * TalkPane.java # 2003-9-28
 * mailto: shenchilang@163.com
 */
package cn.edu.nudt.mice.italk.views;

import java.io.IOException;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IActionBars;

import cn.edu.nudt.mice.italk.Resources;
import cn.edu.nudt.mice.italk.TalkPlugin;
import cn.edu.nudt.mice.italk.content.TextContent;
import cn.edu.nudt.mice.italk.content.TextProvider;
import cn.edu.nudt.mice.italk.preferences.TalkPreferencePage;
import cn.nudt.edu.mice.italk.peer.TalkPeer;
import cn.nudt.edu.mice.italk.peer.TalkServer;

/**
 * @author Shen Rui
 * <code>TalkPane</code>
 */
public class TalkPane extends Composite  {
	public TalkPane(TalkView view, Composite parent){
		super(parent, SWT.NONE);
		_view = view;

		createControls();
		createActions();		
		_server = new TalkServer(2003, _list);
		
		if (TalkPlugin.getDefault()
				.getPreferenceStore()
				.getBoolean(TalkPreferencePage.P_SERVER_AUTO)){
			_listen.setChecked(true); //TODO why this doesn't open the server
			openServer(); 					// so we must open it again here?			
		}
	}
	
	public boolean setFocus(){
		return _provider.getControl().setFocus();
	}

	private void createActions(){
		{		
			class ListenAction extends Action{
				public ListenAction(){
					super("Listen", Action.AS_CHECK_BOX);					
				}
			}
			_listen = new ListenAction(){				
				public void run(){
					if (_server != null){
						if (isChecked()){
							setText("listening");
							openServer();
						}else{
							setText("not listening");
							_server.close();
						}
					}
				}
			};
			_listen.setText("Listen");
			_listen.setImageDescriptor(Resources.IMAGE_LISTEN);
			_listen.setDisabledImageDescriptor(Resources.IMAGE_LISTEN_D);
			_listen.setEnabled(true);
			_listen.setChecked(false);
		}

		IActionBars bars = _view.getViewSite().getActionBars();
		
		bars.getToolBarManager().add(new Separator());
		bars.getToolBarManager().add(_listen);
	}

	public void bind(final TalkPeer peer){
		getDisplay().syncExec(new Runnable(){
			public void run(){
				_bind(peer);
			}
		});
	}
	
	/**
	 * MUST run in safe thread
	 * @safe_thread
	 */
	protected void _bind(TalkPeer peer){
		_content._bind(peer);
		_provider._bind(peer);		
	}
	
	private void openServer(){
		int port = 
			TalkPlugin.getDefault()
				.getPreferenceStore()
				//.getPluginPreferences()
				.getInt(TalkPreferencePage.P_SERVER_PORT);
		_server.setPort(port);
		try {
			_server.open();
		} catch (IOException e) {
			MessageDialog.openError(getShell(), 
					"server open fail", 
					"due to " + e.getClass() + ":" + e.getLocalizedMessage());
			_listen.setChecked(false);
		}
	}
	
	public TalkServer getServer(){
		return _server;
	}
	
	public TalkView getView(){
		return _view;
	}
	
	private void createControls(){
		setLayout(new GridLayout(3, false));
		{
			GridData data = new GridData(GridData.FILL_VERTICAL);
			data.widthHint = 64;
			_list = new PeerList(this);
			_list.setLayoutData(data);
		}
		{
			GridData data = new GridData(GridData.FILL_VERTICAL | GridData.FILL_HORIZONTAL);
			data.widthHint = 100;
			Composite tc = new Composite(this, SWT.NONE);
			tc.setLayoutData(data);
			tc.setLayout(new FillLayout(SWT.VERTICAL));
			_content = new TextContent(_view, tc, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
			_provider = new TextProvider(_view, tc, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		}		
	}
	
	private TalkServer _server;
	private TalkView _view;
	private TextContent _content;
	private TextProvider _provider;
	private PeerList _list;
	private Action _listen;
}
