/*
 * TalkServer.java # 2003-9-28
 * mailto: shenchilang@163.com
 */
package cn.nudt.edu.mice.italk.peer;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.HashMap;

/**
 * @author Shen Rui
 * <code>TalkServer</code>, a socket listener
 */
public class TalkServer implements Runnable{
	/**
	 * construct a server with the port and callback object
	 */
	public TalkServer(int port, TalkServerCallback callback){
		_port = port;
		_callback = callback;
		_peers = new HashMap();
	}
	
	public int getPort(){
		return _port;
	}
	
	public void setPort(int port){
		_port = port;
	}
	
	public void open() throws IOException{
		if (_thread == null){
			_socket = new ServerSocket(_port);
			_thread = new Thread(this);
			_thread.start();
		}
	}
	
	public void close(){
		// be sure to close ONLY once
		if (_thread != null){
			Thread t = _thread;
			_thread = null;
			try {
				_socket.close();
			} catch (IOException e) {
			}
			try {
				t.join();
			} catch (InterruptedException e1) {
			}
		}
	}
	
	void disconnect(TalkPeer peer){
		remove(peer);
		_callback.disconnected(peer);
	}
	
	/**
	 * add a peer to list
	 */
	private void add(TalkPeer peer){
		synchronized(_peers){
			_peers.put(peer.getId(), peer);
		}
	}
	
	/**
	 * remove a peer from list 
	 */
	private void remove(TalkPeer peer){
		synchronized(_peers){
			_peers.remove(peer.getId());
		}
	}
	
	public TalkPeer get(String id){
		synchronized(_peers){
			return (TalkPeer) _peers.get(id);
		}
	}

	public TalkPeer connect(String host, int port) throws IOException{
		TalkPeer peer = new TalkPeer(this, host, port);
		add(peer);
		return peer;
	}
	
	public TalkPeer connect(String host) throws IOException{
		return connect(host, _port);
	}
	/**
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		try{
			while(_thread != null){
				Socket socket = _socket.accept();
				TalkPeer peer = new TalkPeer(this, socket);
				if (_callback.accepted(peer)){
					add(peer);
				}
			}
		}catch(Exception e){
			close();
		}
	}

	private Thread _thread;
	private ServerSocket _socket;
	private TalkServerCallback _callback;
	private int _port;
	private HashMap _peers;
}
